@@ -30,6 +30,10 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from collections import deque
+import logging
+from random import randint
+import time
 
 from gi.repository import GIRepository
 GIRepository.Repository.prepend_search_path('libgd')
@@ -38,24 +42,20 @@ import gi
 gi.require_version('Gst', '1.0')
 gi.require_version('GstAudio', '1.0')
 gi.require_version('GstPbutils', '1.0')
-from gi.repository import Gtk, Gdk, GLib, Gio, GObject, Gst, GstAudio, GstPbutils
+from gi.repository import Gtk, GLib, Gio, GObject, Gst, GstAudio, GstPbutils
 from gettext import gettext as _, ngettext
-from random import randint
-from collections import deque
-from gnomemusic.albumartcache import AlbumArtCache, DefaultIcon, ArtSize
-from gnomemusic.grilo import grilo
-from gnomemusic.playlists import Playlists
-import gnomemusic.utils as utils
-playlists = Playlists.get_default()
-
-from hashlib import md5
-import requests
-import time
-from threading import Thread
 
 from gnomemusic import log
-import logging
+from gnomemusic.albumartcache import Art
+from gnomemusic.grilo import grilo
+from gnomemusic.playlists import Playlists
+from gnomemusic.scrobbler import LastFmScrobbler
+from gnomemusic.widgets.coverstack import CoverStack
+import gnomemusic.utils as utils
+
+
 logger = logging.getLogger(__name__)
+playlists = Playlists.get_default()
 
 
 class RepeatType:
@@ -110,11 +110,6 @@ class Player(GObject.GObject):
         self.playlistField = None
         self.currentTrack = None
         self.currentTrackUri = None
-        scale = parent_window.get_scale_factor()
-        self.cache = AlbumArtCache(scale)
-        self._loading_icon_surface = DefaultIcon(scale).get(
-            DefaultIcon.Type.loading,
-            ArtSize.XSMALL)
         self._missingPluginMessages = []
 
         Gst.init(None)
@@ -146,25 +141,7 @@ class Player(GObject.GObject):
         self.playlist_insert_handler = 0
         self.playlist_delete_handler = 0
 
-        self._check_last_fm()
-
-    @log
-    def _check_last_fm(self):
-        try:
-            self.last_fm = None
-            gi.require_version('Goa', '1.0')
-            from gi.repository import Goa
-            client = Goa.Client.new_sync(None)
-            accounts = client.get_accounts()
-
-            for obj in accounts:
-                account = obj.get_account()
-                if account.props.provider_name == "Last.fm":
-                    self.last_fm = obj.get_oauth2_based()
-                    return
-        except Exception as e:
-            logger.info("Error reading Last.fm credentials: %s" % str(e))
-            self.last_fm = None
+        self._lastfm = LastFmScrobbler()
 
     @log
     def _on_replaygain_setting_changed(self, settings, value):
@@ -602,15 +579,13 @@ class Player(GObject.GObject):
 
         artist = utils.get_artist_name(media)
         self.artistLabel.set_label(artist)
-        self._currentArtist = artist
 
-        self.coverImg.set_from_surface(self._loading_icon_surface)
-        self.cache.lookup(media, ArtSize.XSMALL, self._on_cache_lookup, None)
+        self._cover_stack.update(media)
 
-        self._currentTitle = utils.get_media_title(media)
-        self.titleLabel.set_label(self._currentTitle)
+        title = utils.get_media_title(media)
+        self.titleLabel.set_label(title)
 
-        self._currentTimestamp = int(time.time())
+        self._time_stamp = int(time.time())
 
         url = media.get_url()
         if url != self.player.get_value('current-uri', 0):
@@ -658,8 +633,7 @@ class Player(GObject.GObject):
         return False
 
     @log
-    def _on_cache_lookup(self, surface, data=None):
-        self.coverImg.set_from_surface(surface)
+    def _on_cover_stack_updated(self, klass):
         self.emit('thumbnail-updated')
 
     @log
@@ -681,9 +655,7 @@ class Player(GObject.GObject):
         self.player.set_state(Gst.State.PLAYING)
         self._update_position_callback()
         if media:
-            t = Thread(target=self.update_now_playing_in_lastfm, args=(media.get_url(),))
-            t.setDaemon(True)
-            t.start()
+            self._lastfm.now_playing(media)
         if not self.timeout and self.progressScale.get_realized():
             self._update_timeout()
 
@@ -755,8 +727,6 @@ class Player(GObject.GObject):
     @log
     def set_playlist(self, type, id, model, iter, field,
                      discovery_status_field=11):
-        self.stop()
-
         old_playlist = self.playlist
         if old_playlist != model:
             self.playlist = model
@@ -780,6 +750,8 @@ class Player(GObject.GObject):
             self.emit('playlist-changed')
         self.emit('current-changed')
 
+        GLib.idle_add(self._validate_next_track)
+
     @log
     def running_playlist(self, type, id):
         if type == self.playlistType and id == self.playlistId:
@@ -802,20 +774,14 @@ class Player(GObject.GObject):
         self.songTotalTimeLabel = self._ui.get_object('duration')
         self.titleLabel = self._ui.get_object('title')
         self.artistLabel = self._ui.get_object('artist')
-        self.coverImg = self._ui.get_object('cover')
-        self.coverImg.set_property("width-request", ArtSize.XSMALL.width)
-        self.coverImg.set_property("height-request", ArtSize.XSMALL.height)
+
+        stack = self._ui.get_object('cover')
+        self._cover_stack = CoverStack(stack, Art.Size.XSMALL)
+        self._cover_stack.connect('updated', self._on_cover_stack_updated)
 
         self.duration = self._ui.get_object('duration')
         self.repeatBtnImage = self._ui.get_object('playlistRepeat')
 
-        if Gtk.Settings.get_default().get_property('gtk_application_prefer_dark_theme'):
-            color = Gdk.RGBA(red=1.0, green=1.0, blue=1.0, alpha=1.0)
-        else:
-            color = Gdk.RGBA(red=0.0, green=0.0, blue=0.0, alpha=0.0)
-        self._playImage.override_color(Gtk.StateFlags.ACTIVE, color)
-        self._pauseImage.override_color(Gtk.StateFlags.ACTIVE, color)
-
         self._sync_repeat_image()
 
         self.prevBtn.connect('clicked', self._on_prev_btn_clicked)
@@ -950,67 +916,9 @@ class Player(GObject.GObject):
     def _set_duration(self, duration):
         self.duration = duration
         self.played_seconds = 0
-        self.scrobbled = False
         self.progressScale.set_range(0.0, duration * 60)
 
     @log
-    def scrobble_song(self, url):
-        # Update playlists
-        playlists.update_all_static_playlists()
-
-        if self.last_fm:
-            api_key = self.last_fm.props.client_id
-            sk = self.last_fm.call_get_access_token_sync(None)[0]
-            secret = self.last_fm.props.client_secret
-
-            sig = "api_key%sartist[0]%smethodtrack.scrobblesk%stimestamp[0]%strack[0]%s%s" %\
-                (api_key, self._currentArtist, sk, self._currentTimestamp, self._currentTitle, secret)
-
-            api_sig = md5(sig.encode()).hexdigest()
-            requests_dict = {
-                "api_key": api_key,
-                "method": "track.scrobble",
-                "artist[0]": self._currentArtist,
-                "track[0]": self._currentTitle,
-                "timestamp[0]": self._currentTimestamp,
-                "sk": sk,
-                "api_sig": api_sig
-            }
-            try:
-                r = requests.post("https://ws.audioscrobbler.com/2.0/", requests_dict)
-                if r.status_code != 200:
-                    logger.warn("Failed to scrobble track: %s %s" % (r.status_code, r.reason))
-                    logger.warn(r.text)
-            except Exception as e:
-                logger.warn(e)
-
-    @log
-    def update_now_playing_in_lastfm(self, url):
-        if self.last_fm:
-            api_key = self.last_fm.props.client_id
-            sk = self.last_fm.call_get_access_token_sync(None)[0]
-            secret = self.last_fm.props.client_secret
-
-            sig = "api_key%sartist%smethodtrack.updateNowPlayingsk%strack%s%s" % \
-                (api_key, self._currentArtist, sk, self._currentTitle, secret)
-
-            api_sig = md5(sig.encode()).hexdigest()
-            request_dict = {
-                "api_key": api_key,
-                "method": "track.updateNowPlaying",
-                "artist": self._currentArtist,
-                "track": self._currentTitle,
-                "sk": sk,
-                "api_sig": api_sig
-            }
-            try:
-                r = requests.post("https://ws.audioscrobbler.com/2.0/", request_dict)
-                if r.status_code != 200:
-                    logger.warn("Failed to update currently played track: %s %s" % (r.status_code, r.reason))
-                    logger.warn(r.text)
-            except Exception as e:
-                logger.warn(e)
-
     def _update_position_callback(self):
         position = self.player.query_position(Gst.Format.TIME)[1] / 1000000000
         if position > 0:
@@ -1018,6 +926,7 @@ class Player(GObject.GObject):
         self._update_timeout()
         return False
 
+    @log
     def _update_seconds_callback(self):
         self._on_progress_value_changed(None)
 
@@ -1026,16 +935,18 @@ class Player(GObject.GObject):
             self.played_seconds += self.seconds_period / 1000
             try:
                 percentage = self.played_seconds / self.duration
-                if not self.scrobbled and percentage > 0.4:
+                if (not self._lastfm.scrobbled
+                        and percentage > 0.4):
                     current_media = self.get_current_media()
-                    self.scrobbled = True
                     if current_media:
-                        grilo.bump_play_count(self.get_current_media())
+                        # FIXME: we should not need to update static
+                        # playlists here but removing it may introduce
+                        # a bug. So, we keep it for the time being.
+                        playlists.update_all_static_playlists()
+                        grilo.bump_play_count(current_media)
                         grilo.set_last_played(current_media)
-                        just_played_url = self.get_current_media().get_url()
-                        t = Thread(target=self.scrobble_song, args=(just_played_url,))
-                        t.setDaemon(True)
-                        t.start()
+                        self._lastfm.scrobble(current_media, self._time_stamp)
+
             except Exception as e:
                 logger.warn("Error: %s, %s", e.__class__, e)
         return True
@@ -1197,5 +1108,4 @@ class SelectionToolbar():
         self._ui.add_from_resource('/org/gnome/Music/SelectionToolbar.ui')
         self.actionbar = self._ui.get_object('actionbar')
         self._add_to_playlist_button = self._ui.get_object('button1')
-        self._remove_from_playlist_button = self._ui.get_object('button2')
         self.actionbar.set_visible(False)
