@@ -606,6 +606,29 @@ class Query():
         }
         return query
 
+    @staticmethod
+    def change_song_position(playlist_id, song_id, new_position):
+        query = """
+    INSERT OR REPLACE {
+        ?entry
+            nfo:listPosition %(position)s
+    }
+    WHERE {
+        ?playlist a nmm:Playlist ;
+            a nfo:MediaList ;
+            nfo:hasMediaFileListEntry ?entry .
+        FILTER (
+            tracker:id(?playlist) = %(playlist_id)s &&
+            tracker:id(?entry) = %(song_id)s
+        )
+    }
+    """.replace("\n", " ").strip() % {
+            'playlist_id': playlist_id,
+            'song_id': song_id,
+            'position': float(new_position)
+        }
+        return query
+
     @staticmethod
     def get_playlist_with_id(playlist_id):
         query = """
@@ -936,26 +959,21 @@ class Query():
     @staticmethod
     def get_songs_with_artist_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = """?performer fts:match '"nmm:artistName" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
+        query = """?song nmm:performer [ fts:match '%(name)s*' ] . """.replace('\n',' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_album_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = """?album fts:match '"nie:title" : %(name)s*' . """.replace('\n', ' ').strip() % {'name': name}
+        query = """?song nmm:musicAlbum [ fts:match '%(name)s*' ] . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
     @staticmethod
     def get_songs_with_composer_match(name):
         name = Tracker.sparql_escape_string(name)
-        query = """
-            ?song nmm:composer ?composer .
-            ?composer fts:match '"nmm:artistName" : %(name)s*' .
-        """.replace('\n', ' ').strip() % {
-            'name': name
-        }
+        query = """?song nmm:composer [ fts:match '%(name)s*' ] . """.replace('\n', ' ').strip() % {'name': name}
 
         return Query.songs(query)
 
