@@ -26,11 +26,12 @@ from gettext import gettext as _
 from gi.repository import GLib, GObject, Gtk, Gdk
 
 from gnomemusic import log
-from gnomemusic.albumartcache import ArtSize
+from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
 from gnomemusic.toolbar import ToolbarState
 from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.albumwidget import AlbumWidget
+from gnomemusic.widgets.coverstack import CoverStack
 import gnomemusic.utils as utils
 
 
@@ -50,6 +51,7 @@ class AlbumsView(BaseView):
         self.items_selected = []
         self.items_selected_callback = None
         self._add_list_renderers()
+        self._init = True
 
     @log
     def _on_changes_pending(self, data=None):
@@ -119,7 +121,7 @@ class AlbumsView(BaseView):
 
     @log
     def populate(self):
-        self._window.push_loading_notification()
+        self._window.notifications_popup.push_loading()
         grilo.populate_albums(self._offset, self._add_item)
 
     @log
@@ -145,8 +147,8 @@ class AlbumsView(BaseView):
             child = self._create_album_item(item)
             self._view.add(child)
         elif remaining == 0:
-                self._window.pop_loading_notification()
-                self._view.show()
+            self._window.notifications_popup.pop_loading()
+            self._view.show()
 
     def _create_album_item(self, item):
         artist = utils.get_artist_name(item)
@@ -156,7 +158,7 @@ class AlbumsView(BaseView):
             '/org/gnome/Music/AlbumCover.ui')
 
         child = Gtk.FlowBoxChild()
-        child.image = builder.get_object('image')
+        child.stack = builder.get_object('stack')
         child.check = builder.get_object('check')
         child.title = builder.get_object('title')
         child.subtitle = builder.get_object('subtitle')
@@ -166,13 +168,6 @@ class AlbumsView(BaseView):
         child.title.set_label(title)
         child.subtitle.set_label(artist)
 
-        child.image.set_from_surface(self._loading_icon_surface)
-        # In the case of off-sized icons (eg. provided in the soundfile)
-        # keep the size request equal to all other icons to get proper
-        # alignment with GtkFlowBox.
-        child.image.set_property("width-request", ArtSize.MEDIUM.width)
-        child.image.set_property("height-request", ArtSize.MEDIUM.height)
-
         child.events.add_events(Gdk.EventMask.TOUCH_MASK)
 
         child.events.connect('button-release-event',
@@ -189,7 +184,8 @@ class AlbumsView(BaseView):
         child.add(builder.get_object('main_box'))
         child.show()
 
-        self._cache.lookup(item, ArtSize.MEDIUM, self._on_lookup_ready, child)
+        cover_stack = CoverStack(child.stack, Art.Size.MEDIUM)
+        cover_stack.update(item)
 
         return child
 
@@ -200,9 +196,6 @@ class AlbumsView(BaseView):
             if self.selection_mode:
                 child.check.set_active(True)
 
-    def _on_lookup_ready(self, icon, child):
-        child.image.set_from_surface(icon)
-
     @log
     def _on_child_toggled(self, check, pspec, child):
         if (check.get_active()
