@@ -23,9 +23,9 @@
 # delete this exception statement from your version.
 
 from gettext import gettext as _
-from gi.repository import Gd, Gdk, GdkPixbuf, GObject, Grl, Gtk, Pango
+from gi.repository import Gd, GdkPixbuf, GObject, Grl, Gtk, Pango
 
-from gnomemusic.albumartcache import DefaultIcon, ArtSize
+from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
 from gnomemusic import log
 from gnomemusic.player import DiscoveryStatus
@@ -54,13 +54,6 @@ class SearchView(BaseView):
     def __init__(self, window, player):
         super().__init__('search', None, window, Gd.MainViewType.LIST)
 
-        scale = self.get_scale_factor()
-        loading_icon_surface = DefaultIcon(scale).get(
-            DefaultIcon.Type.loading, ArtSize.SMALL)
-        self._loading_icon = Gdk.pixbuf_get_from_surface(
-            loading_icon_surface, 0, 0, loading_icon_surface.get_width(),
-            loading_icon_surface.get_height())
-
         self._add_list_renderers()
         self.player = player
         self._head_iters = [None, None, None, None]
@@ -178,14 +171,12 @@ class SearchView(BaseView):
 
         artist = utils.get_artist_name(item)
         album = utils.get_album_title(item)
-        composer = item.get_composer()
 
         key = '%s-%s' % (artist, album)
         if key not in self._albums:
             self._albums[key] = Grl.Media()
             self._albums[key].set_title(album)
             self._albums[key].add_artist(artist)
-            self._albums[key].set_composer(composer)
             self._albums[key].set_source(source.get_id())
             self._albums[key].songs = []
             self._add_item(
@@ -196,6 +187,13 @@ class SearchView(BaseView):
         self._albums[key].songs.append(item)
         self._add_item(source, None, item, 0, [self.model, 'song'])
 
+    @log
+    def _retrieval_finished(self, klass, model, _iter):
+        if not model[_iter][13]:
+            return
+
+        model[_iter][13] = klass.surface
+
     @log
     def _add_item(self, source, param, item, remaining=0, data=None):
         if data is None:
@@ -223,7 +221,7 @@ class SearchView(BaseView):
             self.previous_view = self._window.prev_view
 
         if remaining == 0:
-            self._window.pop_loading_notification()
+            self._window.notifications_popup.pop_loading()
             self._view.show()
 
         if not item or model != self.model:
@@ -233,8 +231,6 @@ class SearchView(BaseView):
         title = utils.get_media_title(item)
         item.set_title(title)
         artist = utils.get_artist_name(item)
-        # FIXME: Can't be None in treemodel
-        composer = item.get_composer() or ""
 
         group = 3
         try:
@@ -242,17 +238,12 @@ class SearchView(BaseView):
         except:
             pass
 
-        # FIXME: HiDPI icon lookups return a surface that can't be
-        # scaled by GdkPixbuf, so it results in a * scale factor sized
-        # icon for the search view.
         _iter = None
         if category == 'album':
             _iter = self.model.insert_with_values(
-                self._head_iters[group], -1, [0, 2, 3, 4, 5, 9, 11, 13],
-                [str(item.get_id()), title, artist, self._loading_icon, item,
-                 2, category, composer])
-            self._cache.lookup(
-                item, ArtSize.SMALL, self._on_lookup_ready, _iter)
+                self._head_iters[group], -1, [0, 2, 3, 5, 9, 11],
+                [str(item.get_id()), title, artist, item, 2,
+                 category])
         elif category == 'song':
             # FIXME: source specific hack
             if source.get_id() != 'grl-tracker-source':
@@ -260,26 +251,30 @@ class SearchView(BaseView):
             else:
                 fav = item.get_favourite()
             _iter = self.model.insert_with_values(
-                self._head_iters[group], -1, [0, 2, 3, 4, 5, 9, 11, 13],
-                [str(item.get_id()), title, artist, self._loading_icon, item,
-                 fav, category, composer])
-            self._cache.lookup(
-                item, ArtSize.SMALL, self._on_lookup_ready, _iter)
+                self._head_iters[group], -1, [0, 2, 3, 5, 9, 11],
+                [str(item.get_id()), title, artist, item, fav,
+                 category])
         else:
             if not artist.casefold() in self._artists:
                 _iter = self.model.insert_with_values(
-                    self._head_iters[group], -1, [0, 2, 4, 5, 9, 11, 13],
-                    [str(item.get_id()), artist, self._loading_icon, item, 2,
-                     category, composer])
-                self._cache.lookup(
-                    item, ArtSize.SMALL, self._on_lookup_ready, _iter)
+                    self._head_iters[group], -1, [0, 2, 5, 9, 11],
+                    [str(item.get_id()), artist, item, 2,
+                     category])
                 self._artists[artist.casefold()] = {
                     'iter': _iter,
                     'albums': []
                 }
-
             self._artists[artist.casefold()]['albums'].append(item)
 
+        # FIXME: Figure out why iter can be None here, seems illogical.
+        if _iter is not None:
+            scale = self._view.get_scale_factor()
+            art = Art(Art.Size.SMALL, item, scale)
+            self.model[_iter][13] = art.surface
+            art.connect(
+                'finished', self._retrieval_finished, self.model, _iter)
+            art.lookup()
+
         if self.model.iter_n_children(self._head_iters[group]) == 1:
             path = self.model.get_path(self._head_iters[group])
             path = self._filter_model.convert_child_path_to_path(path)
@@ -299,13 +294,31 @@ class SearchView(BaseView):
             title_renderer, self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        self._star_handler.add_star_renderers(list_widget, cols[0])
+        # Add our own surface renderer, instead of the one provided by
+        # Gd. This avoids us having to set the model to a cairo.Surface
+        # which is currently not a working solution in pygobject.
+        # https://gitlab.gnome.org/GNOME/pygobject/issues/155
+        pixbuf_renderer = Gtk.CellRendererPixbuf(
+            xalign=0.5, yalign=0.5, xpad=12, ypad=2)
+        list_widget.add_renderer(
+            pixbuf_renderer, self._on_list_widget_pixbuf_renderer, None)
+        cols[0].add_attribute(pixbuf_renderer, 'surface', 13)
 
+        self._star_handler.add_star_renderers(list_widget, cols[0])
         cells = cols[0].get_cells()
         cols[0].reorder(cells[0], -1)
+        cols[0].reorder(cells[4], 1)
         cols[0].set_cell_data_func(
             cells[0], self._on_list_widget_selection_render, None)
 
+    @log
+    def _on_list_widget_pixbuf_renderer(self, col, cell, model, _iter, data):
+        if not model[_iter][13]:
+            return
+
+        cell.set_property("surface", model[_iter][13])
+
+    @log
     def _on_list_widget_selection_render(self, col, cell, model, _iter, data):
         if (self._view.get_selection_mode()
                 and model.iter_parent(_iter) is not None):
@@ -313,16 +326,18 @@ class SearchView(BaseView):
         else:
             cell.set_visible(False)
 
+    @log
     def _on_list_widget_title_render(self, col, cell, model, _iter, data):
         cells = col.get_cells()
-        cells[0].set_visible(model.iter_parent(_iter) is not None)
+        cells[0].set_visible(False)
         cells[1].set_visible(model.iter_parent(_iter) is not None)
-        cells[2].set_visible(model.iter_parent(_iter) is None)
+        cells[2].set_visible(model.iter_parent(_iter) is not None)
+        cells[3].set_visible(model.iter_parent(_iter) is None)
 
     @log
     def populate(self):
         self._init = True
-        self._window.push_loading_notification()
+        self._window.notifications_popup.push_loading()
         self._header_bar.set_state(ToolbarState.MAIN)
 
     @log
@@ -460,7 +475,7 @@ class SearchView(BaseView):
             GObject.TYPE_STRING,
             GObject.TYPE_STRING,    # item title or header text
             GObject.TYPE_STRING,    # artist for albums and songs
-            GdkPixbuf.Pixbuf,       # album art
+            GdkPixbuf.Pixbuf,       # Gd placeholder album art
             GObject.TYPE_OBJECT,    # item
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_INT,
@@ -469,7 +484,7 @@ class SearchView(BaseView):
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_STRING,    # type
             GObject.TYPE_INT,
-            GObject.TYPE_STRING,    # composer
+            object                  # album art surface
         )
 
         self._filter_model = self.model.filter_new(None)
